/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2012 by Tidalwave s.a.s. (http://tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.integritychecker.archive.impl;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.NotThreadSafe;
import java.util.Date;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;
import java.text.SimpleDateFormat;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.nio.file.Path;
import it.tidalwave.messagebus.annotation.ListensTo;
import it.tidalwave.actor.MessageSupport;
import it.tidalwave.actor.annotation.Actor;
import it.tidalwave.actor.annotation.Message;
import it.tidalwave.actor.io.filescan.FileDamageDetectedMessage;
import it.tidalwave.actor.io.filescan.FileDiscoveredMessage;
import it.tidalwave.integritychecker.fingerprint.Fingerprint;
import it.tidalwave.integritychecker.fingerprint.FingerprintComputedMessage;
import it.tidalwave.integritychecker.archive.OpenScanRequest;
import lombok.Cleanup;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;
import static java.util.concurrent.TimeUnit.*;

/***********************************************************************************************************************
 * 
 * @stereotype Actor
 * 
 * An implementation of {@link PersistenceManagerActorSupport} which stores data to a flat file.
 * WARNING: functions in this actor are being moved to ScanArchiveActor, so this class is legacy.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Actor(threadSafe=false) @NotThreadSafe @Slf4j
public class LegacyPersistenceManagerActor 
  {
    @Message(outOfBand=true, daemon=true) @ToString
    static class FlushRequestMessage extends MessageSupport
      {
      }
    
    private static final String PERSISTENCE_FOLDER_NAME = ".SolidBlue";
    
    private final Map<Path, Object> map = new TreeMap<>(); 
    
    private File persistenceFile;
    
    private boolean flushPending;
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    void onOpenScanRequest (final @ListensTo @Nonnull OpenScanRequest openScanRequest)
      throws IOException
      {
        log.info("onOpenScanRequest({})", openScanRequest);  
        File parentFolder = openScanRequest.getFolder().toFile();
        
        if (!parentFolder.canWrite())
          {  
            parentFolder = new File(System.getProperty("user.home"));   
          }
        
        final File folder = new File(parentFolder, PERSISTENCE_FOLDER_NAME);
        folder.mkdirs();
        
        if (!folder.exists())
          {
            throw new IOException("Cannot create " + folder.getAbsolutePath());  
          }
        
        final String fileName = new SimpleDateFormat("'fingerprints-'yyyyMMdd_HHmm'.txt'").format(new Date());
        persistenceFile = new File(folder, fileName);
        map.clear();
        log.info("Persisting to {}", persistenceFile.getAbsolutePath());
      }
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    void onFileDiscovered (final @ListensTo @Nonnull FileDiscoveredMessage message)
      {
        log.debug("onFileDiscovered({})", message);
        final Path name = indexFor(message.getPath());
        
        if (!map.containsKey(name)) // message sequentiality is not guaranteed, fingerprint could have been already computed
          {
            map.put(name, "unavailable");  
            requestFlush();
          }
      }

    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    void onFingerprintComputed (final @ListensTo @Nonnull FingerprintComputedMessage message)
      {
        log.debug("onFingerprintComputed({})", message);
        map.put(indexFor(message.getPath()), message.getFingerprint());
        requestFlush();
      }

    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    void onDamagedFileDetected (final @ListensTo @Nonnull FileDamageDetectedMessage message)
      {
        log.debug("onDamagedFileDetected({})", message);
        map.put(indexFor(message.getPath()), "broken: " + message.getCause());  
        requestFlush();
      }
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    void onFlushRequested (final @ListensTo @Nonnull FlushRequestMessage message)
      throws InterruptedException, IOException
      {
        log.debug("onFlushRequested()");
        flushPending = false;
        final @Cleanup PrintWriter pw = new PrintWriter(persistenceFile);

        for (final Entry<Path, Object> entry : map.entrySet())
          {
            if (entry.getValue() instanceof Fingerprint)
              {
                final Fingerprint fingerprint = (Fingerprint)entry.getValue();
                pw.printf("%s(%s)=%s\n", fingerprint.getType(), entry.getKey(), fingerprint.getValue());
              }
            else
              {
                pw.printf("%s=%s\n", entry.getKey(), entry.getValue());
              }
          }

        pw.close();
      }
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    private void requestFlush()
      {
        if (!flushPending)
          {
            log.debug(">>>> requesting flush...");
            new FlushRequestMessage().sendLater(5, SECONDS);
            flushPending = true;
          }
      }
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    @Nonnull
    private Path indexFor (final @Nonnull Path path)
      {
        return path.getFileName();
      }
  }
